@@ -1,7 +1,5 @@
-import docker
-from dagster_docker.utils import DOCKER_CONFIG_SCHEMA, validate_docker_config, validate_docker_image
-
 import dagster._check as check
+import docker
 from dagster._core.launcher.base import (
     CheckRunHealthResult,
     LaunchRunContext,
@@ -9,12 +7,14 @@ from dagster._core.launcher.base import (
     RunLauncher,
     WorkerStatus,
 )
-from dagster._core.storage.pipeline_run import PipelineRun
+from dagster._core.storage.pipeline_run import DagsterRun
 from dagster._core.storage.tags import DOCKER_IMAGE_TAG
 from dagster._core.utils import parse_env_var
 from dagster._grpc.types import ExecuteRunArgs, ResumeRunArgs
 from dagster._serdes import ConfigurableClass
 
+from dagster_docker.utils import DOCKER_CONFIG_SCHEMA, validate_docker_config, validate_docker_image
+
 from .container_context import DockerContainerContext
 
 DOCKER_CONTAINER_ID_TAG = "docker/container_id"
@@ -65,7 +65,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
     def from_config_value(inst_data, config_value):
         return DockerRunLauncher(inst_data=inst_data, **config_value)
 
-    def get_container_context(self, pipeline_run: PipelineRun) -> DockerContainerContext:
+    def get_container_context(self, pipeline_run: DagsterRun) -> DockerContainerContext:
         return DockerContainerContext.create_for_run(pipeline_run, self)
 
     def _get_client(self, container_context: DockerContainerContext):
@@ -93,6 +93,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
     def _launch_container_with_command(self, run, docker_image, command):
         container_context = self.get_container_context(run)
         docker_env = dict([parse_env_var(env_var) for env_var in container_context.env_vars])
+        docker_env["DAGSTER_RUN_JOB_NAME"] = run.job_name
 
         client = self._get_client(container_context)
 
@@ -123,9 +124,11 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
                 network.connect(container)
 
         self._instance.report_engine_event(
-            message="Launching run in a new container {container_id} with image {docker_image}".format(
-                container_id=container.id,
-                docker_image=docker_image,
+            message=(
+                "Launching run in a new container {container_id} with image {docker_image}".format(
+                    container_id=container.id,
+                    docker_image=docker_image,
+                )
             ),
             pipeline_run=run,
             cls=self.__class__,
@@ -206,9 +209,9 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
     def supports_check_run_worker_health(self):
         return True
 
-    def check_run_worker_health(self, run: PipelineRun):
+    def check_run_worker_health(self, run: DagsterRun):
         container = self._get_container(run)
-        if container == None:
+        if container is None:
             return CheckRunHealthResult(WorkerStatus.NOT_FOUND)
         if container.status == "running":
             return CheckRunHealthResult(WorkerStatus.RUNNING)
